﻿/*
 * ##############################################
 * CalcVariables
 * Main part of the QuickCalculator
 * ##############################################
 * 
 * You may copy and distribute this code and the 
 * whole program as you want.
 * This code is not meant to be sold.
 * 
 * Written by Philipp Erler
 * Contact me for feedback, new version or other causes:
 * ph.erler@gmx.net
 * 
 * Version info:
 * Last change: 2011 | 09 | 01
 * .net-Framework: V. 4.0
 * 
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CalculationE
{
    static class CalcVariables
    {
        private const int MAX_Vars = 100;

        public struct variable                                         //input:   abc = 3.14159
        {
            public string name;                                         //example: abc
            public double value;                                        //example: 3.14159

            public variable(string name, double val)
            {
                this.name = name;
                this.value = val;
            }
        }

        static private List<variable> calcVars = new List<variable>();

        static CalcVariables()
        {
            //set standard vars
            Set_Var("ans", 0);
            Set_Var("rand", CalcGlobals.Randoms.NextDouble());                 //set random number
            //Set_Var("pi", CalcGlobals.PI);
            //Set_Var("e", CalcGlobals.Euler);
            //Set_Var("gearth", CalcGlobals.GravEarth);
            //Set_Var("G", CalcGlobals.Grav);
            //Set_Var("c", CalcGlobals.LightSpeed);
            //Set_Var("efield", CalcGlobals.ElField);
            //Set_Var("euma", CalcGlobals.EulerMascharoni);
            //Set_Var("magfield", CalcGlobals.MagField);
            //Set_Var("elecharge", CalcGlobals.ElecCharge);
        }

        //Save calc var in array.
        //Note: Can overwrite existing var value. Do NOT use while calculation is running!
        static public void Set_Var(string name, double value)
        {
            int varExistIndex = -1;

            //catch no name
            if (name == "")
            {
                throw new Exception("No name given");
            }

            //if name begins wit "-"
            if (name.StartsWith("-") && name.Length > 1)
            {
                name = name.Substring(1);   //remove "-" from name
                value = -value;             //invert value
            }

            //catch numeric names
            {
                double d = 0.0;
                bool success;
                success = Double.TryParse(name, out d);
                if (success) throw new Exception("Variable names cannot be numbers");
            }

            //number must not contain operators
            for (int i = 0; i <= CalcOperators.opCompareStr.GetUpperBound(0); ++i)
            {
                if (name.IndexOf(CalcOperators.opCompareStr[i]) > -1)
                    throw new Exception("Operators cannot be part of the name of a variable");
            }

            //TODO: without lowercase
            //lower case
            name = name.ToLower();

            //get index of next empty var or existing var of this name
            for (int i = 0; i < calcVars.Count; ++i)
            {
                if (calcVars[i].name == name)
                {
                    varExistIndex = i;
                    break;
                }
            }

            //set name and value
            if (varExistIndex == -1)
                calcVars.Add(new variable(name, value));
            else
                calcVars[varExistIndex] = new variable(name, value);

        }

        /// <summary>
        /// Returns number of assigned functions
        /// </summary>
        /// <returns></returns>
        static public int Count()
        { return calcVars.Count(); }

        //get calc var out of array
        static public double Get_Var(string name)
        {
            int varIndex = -1;

            if (name == null)
                throw new Exception("No name given");

            if (name == "")
                throw new Exception("No name given");
            else if (name == "rand")
                    Set_Var("rand", CalcGlobals.Randoms.NextDouble());

            name = name.ToLower();

            for (int i = 0; i <= calcVars.Count - 1; ++i)
            {
                if (calcVars[i].name.ToLower() == name)
                {
                    varIndex = i;
                    break;
                }
            }

            if (varIndex != -1)
                return calcVars[varIndex].value;
            else
            {
                throw new Exception("Variable " + name + " not found");
            }
        }

        //get calc var out of array
        static public double Get_Var(int index)
        {
            if (index < 0 || index >= calcVars.Count())
                throw new Exception("Invalid Index");

            if (calcVars[index].name == "rand")
                Set_Var("rand", CalcGlobals.Randoms.NextDouble());

            return calcVars[index].value;
        }

        /// <summary>
        /// Get calc var index. Return -1 if not existing.
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        static public int Get_Var_Index(string name)
        {
            int varIndex = -1;

            name = name.ToLower();

            for (int i = 0; i <= calcVars.Count - 1; ++i)
            {
                if (calcVars[i].name == name)
                {
                    varIndex = i;
                    break;
                }
            }

            return varIndex;
        }

        //remove calc var
        static public void Del_Var(string name)
        {
            int varIndex = -1;

            if (name == "")
            {
                throw new Exception("No name given");
            }

            name = name.ToLower();

            for (int i = 0; i <= calcVars.Count - 1; ++i)
            {
                if (calcVars[i].name == name)
                {
                    varIndex = i;
                    break;
                }
            }

            if (varIndex == -1)
                throw new Exception("Variable " + name + " not found");
            else
                calcVars.RemoveAt(varIndex);

        }

        //remove calc var
        static public void Del_Var(int index)
        {
            if (index < 0 || index > calcVars.Count - 1)
                return;
            else
                calcVars.RemoveAt(index);

        }

        //remove all calc vars
        static public void Del_Vars()
        {
            calcVars.Clear();
        }

        //return list containing all functions and expressions
        static public string Print_Vars()
        {
            string varlist = "Variable list:";

            for (int i = 0; i <= calcVars.Count - 1; ++i)
            {
                varlist = varlist + (char)13 + (char)10 + calcVars[i].name + " = " + calcVars[i].value;
            }

            return varlist;
        }

        //return 1 variable and its value
        static public string Print_Var(int VarIndex)
        {
            string varNameValue = VarIndex.ToString() + ": ";

            if (VarIndex < 0 || VarIndex > calcVars.Count - 1)
                return "";

            varNameValue = varNameValue + calcVars[VarIndex].name + " = " + calcVars[VarIndex].value;

            return varNameValue;
        }

        static public string Get_Var_Name(int Index)
        { return calcVars[Index].name; }
    }
}
