﻿/*
 * ##############################################
 * DrawFunctions-Form
 * Part of the QuickCalculator
 * ##############################################
 * 
 * You may copy and distribute this code and the 
 * whole program as you want.
 * This code is not meant to be sold.
 * 
 * Written by Philipp Erler
 * Contact me for feedback, new version or other causes:
 * ph.erler@gmx.net
 * 
 * Version info:
 * Last change: 2013 | 02 | 03
 * 
 * .net-Framework: V. 4.0
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CalculationE;
using FunctionDrawing;

namespace QuickCalculator
{
    public partial class frmDrawFunction : Form
    {
        FunctionDraw fctDraw;

        List<Pen> fctPens = new List<Pen>();                                    //pens for function graphs

        bool selChangeByCode = false;

        //constructor
        public frmDrawFunction()
        {
            InitializeComponent();

            fctDraw = new FunctionDraw(picDraw.Width, picDraw.Height);
            fctDraw.Set_Progress_Bar(this.toolStripProgressBar);
            fctDraw.Set_Fct_Pens(fctPens);

            Actualize_Function_Data();
            fctDraw.Draw_Scale(picDraw.CreateGraphics());
        }

        //destructor
        ~frmDrawFunction()
        {
            Dispose(true);
        }

        #region Script_Function_Targets

        /// <summary>
        /// Select and draw these functions.
        /// </summary>
        /// <param name="fcts"></param>
        public void Select_Fcts(string[] fcts)
        {
            Actualize_Function_Data();
            selChangeByCode = true;
            dataGridFct.ClearSelection();
            for (int i = 0; i <= fcts.GetUpperBound(0); ++i)
            {
                for (int n = 0; n < fctDraw.fctNames.Count; ++n)
                {
                    if (String.Equals(fctDraw.fctNames[n], fcts[i], StringComparison.CurrentCultureIgnoreCase))
                    {
                        //dataGridFct.CurrentRow;
                        selChangeByCode = true;
                        dataGridFct.Rows[n].Selected = true;
                        break;
                    }
                }
                //Here: Did not find match for fcts[i]
            }
            Actualize_Function_Selection();
        }

        public void Set_Origin(int x, int y)
        {
            fctDraw.Set_Origin_Y(x);
            fctDraw.Set_Origin_Y(y);
            txtOMovX.Text = x.ToString();
            txtOMovY.Text = y.ToString();
            Fct_Redraw();
        }

        public void Set_Scale(double x, double y)
        {
            fctDraw.Set_Scale_X(x);
            fctDraw.Set_Scale_Y(y);
            txtScaleX.Text = x.ToString();
            txtScaleY.Text = y.ToString();
            Fct_Redraw();
        }

        public void Fct_Redraw()
        {
            Actualize_OriginAndScale();
            fctDraw.Calc_Functions();
            picDraw.Invalidate();
        }

        #endregion

        private void Actualize_Function_Data()
        {
            fctDraw.Actualize_Functions();
            Actualize_Function_ListBox();
            Actualize_Function_Selection();
        }

        private void Actualize_Function_Selection()
        {
            for (int i = 0; i < dataGridFct.Rows.Count; ++i)
            {
                fctDraw.fctSelect[i] = dataGridFct.Rows[i].Selected;
            }

            fctDraw.Calc_Functions();
            picDraw.Invalidate();
        }

        //add functions from calcengine to listbox
        public void Actualize_Function_ListBox()
        {
            Random rand = new Random();

            selChangeByCode = true;
            dataGridFct.Rows.Clear();
            fctPens.Clear();
            for (int i = 0; i < fctDraw.fctPrint.Count(); ++i)
            {
                if (fctDraw.fctPrint[i] != "")
                {
                    selChangeByCode = true;
                    dataGridFct.Rows.Add();
                    dataGridFct.Rows[i].Cells[0].Value = fctDraw.fctPrint[i];

                    //set function draw color
                    switch (i)
                    {
                        case 0:
                            dataGridFct.Rows[i].Cells[0].Style.ForeColor = Color.FromKnownColor(KnownColor.Red);
                            break;
                        case 1:
                            dataGridFct.Rows[i].Cells[0].Style.ForeColor = Color.FromKnownColor(KnownColor.Blue);
                            break;
                        case 2:
                            dataGridFct.Rows[i].Cells[0].Style.ForeColor = Color.FromKnownColor(KnownColor.Yellow);
                            break;
                        case 3:
                            dataGridFct.Rows[i].Cells[0].Style.ForeColor = Color.FromKnownColor(KnownColor.Green);
                            break;
                        case 4:
                            dataGridFct.Rows[i].Cells[0].Style.ForeColor = Color.FromKnownColor(KnownColor.Orange);
                            break;
                        case 5:
                            dataGridFct.Rows[i].Cells[0].Style.ForeColor = Color.FromKnownColor(KnownColor.Purple);
                            break;
                        default:
                            dataGridFct.Rows[i].Cells[0].Style.ForeColor =
                                Color.FromArgb(rand.Next(256), rand.Next(256), rand.Next(256));
                            break;
                    }
                    dataGridFct.Rows[i].Cells[1].Style.BackColor = dataGridFct.Rows[i].Cells[0].Style.ForeColor;
                    fctPens.Add(new Pen(dataGridFct.Rows[i].Cells[0].Style.ForeColor));
                }
                else
                    break;
            }

            //set selection as saved
            for (int i = 0; i < fctDraw.fctSelect.Count; ++i)
            {
                selChangeByCode = true;
                dataGridFct.Rows[i].Selected = fctDraw.fctSelect[i];
            }
            selChangeByCode = false;
        }

        private void dataGridFct_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            //set color of function font
            if (e.ColumnIndex == 1)
            {
                if (colorDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    dataGridFct.Rows[e.RowIndex].Cells[0].Style.ForeColor = colorDialog.Color;
                    fctPens[e.RowIndex] = new Pen(dataGridFct.Rows[e.RowIndex].Cells[0].Style.ForeColor);
                }

                actualizeToolStripMenuItem.PerformClick();
            }
        }

        private void dataGridFct_SelectionChanged(object sender, EventArgs e)
        {
            if (!selChangeByCode)
                Actualize_Function_Selection();
            else
                selChangeByCode = false;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        { Application.Exit(); }

        private void closeWindowToolStripMenuItem_Click(object sender, EventArgs e)
        { this.Hide(); }

        //actualize everything except functions in listbox
        public void actualizeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Actualize_OriginAndScale();
            fctDraw.Calc_Functions();
            picDraw.Invalidate();
        }

        //mouse move over picturebox ~ drawing screen
        private void picDraw_MouseMove(object sender, MouseEventArgs e)
        {
            //get mouse coords für state bar
            toolStripMouseX.Text = fctDraw.Get_Mouse_Coord_X(e.X);
            toolStripMouseY.Text = fctDraw.Get_Mouse_Coord_Y(e.Y);

            fctDraw.Actualize_Zoom_Box(e.X, e.Y);

            picDraw.Invalidate();
        }

        //get all view modificators out of the textboxes
        private void Actualize_OriginAndScale()
        {
            fctDraw.Set_Scale_X(System.Convert.ToDouble(txtScaleX.Text));
            fctDraw.Set_Scale_Y(System.Convert.ToDouble(txtScaleY.Text));
            fctDraw.Set_Origin_X(System.Convert.ToInt32(txtOMovX.Text));
            fctDraw.Set_Origin_Y(System.Convert.ToInt32(txtOMovY.Text));
            fctDraw.Set_Accurracy(System.Convert.ToInt32(txtaccuracy.Text));
        }

        #region Change_View_Param
        private void butScaleXIn_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Scale_X(fctDraw.Get_Scale_X() * 1.1);
            if (fctDraw.Get_Scale_X() > 10)
                fctDraw.Set_Scale_X(Math.Round(fctDraw.Get_Scale_X()));
            txtScaleX.Text = fctDraw.Get_Scale_X().ToString();
        }

        private void butScaleXDe_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Scale_X(fctDraw.Get_Scale_X() * 0.9);
            if (fctDraw.Get_Scale_X() > 10)
                fctDraw.Set_Scale_X(Math.Round(fctDraw.Get_Scale_X()));
            txtScaleX.Text = fctDraw.Get_Scale_X().ToString();
        }

        private void butScaleYIn_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Scale_Y(fctDraw.Get_Scale_Y() * 1.1);
            if (fctDraw.Get_Scale_Y() > 10)
                fctDraw.Set_Scale_Y(Math.Round(fctDraw.Get_Scale_Y()));
            txtScaleY.Text = fctDraw.Get_Scale_Y().ToString();
        }

        private void butScaleYDe_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Scale_Y(fctDraw.Get_Scale_Y() * 0.9);
            if (fctDraw.Get_Scale_Y() > 10)
                fctDraw.Set_Scale_Y(Math.Round(fctDraw.Get_Scale_Y()));
            txtScaleY.Text = fctDraw.Get_Scale_Y().ToString();
        }

        private void butOriginXIn_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Origin_X(fctDraw.Get_Origin_X() + 50);
            txtOMovX.Text = fctDraw.Get_Origin_X().ToString();
        }

        private void butOriginXDe_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Origin_X(fctDraw.Get_Origin_X() - 50);
            txtOMovX.Text = fctDraw.Get_Origin_X().ToString();
        }

        private void butOriginYIn_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Origin_Y(fctDraw.Get_Origin_X() + 50);
            txtOMovY.Text = fctDraw.Get_Origin_Y().ToString();
        }

        private void butOriginYDe_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Origin_X(fctDraw.Get_Origin_Y() - 50);
            txtOMovY.Text = fctDraw.Get_Origin_Y().ToString();
        }

        private void butAccurIn_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Accurracy(fctDraw.Get_Accurracy() + 1);
            txtaccuracy.Text = fctDraw.Get_Accurracy().ToString();
        }

        private void butAccurDe_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Accurracy(fctDraw.Get_Accurracy() - 1);
            txtaccuracy.Text = fctDraw.Get_Accurracy().ToString();
        }

        private void txtaccuracy_TextChanged(object sender, EventArgs e)
        {
            try
            { fctDraw.Set_Accurracy(System.Convert.ToInt32(txtaccuracy.Text)); }
            catch (Exception)
            {
                fctDraw.Set_Accurracy(5);
                txtaccuracy.Text = "5";
            }
        }

        private void txtOMovY_TextChanged(object sender, EventArgs e)
        {
            try
            { fctDraw.Set_Origin_Y(System.Convert.ToInt32(txtOMovY.Text)); }
            catch (Exception)
            {
                fctDraw.Set_Origin_Y(0);
                txtOMovY.Text = "0";
            }
            //redraw without invalid function
            picDraw.Invalidate();
        }

        private void txtOMovX_TextChanged(object sender, EventArgs e)
        {
            try
            { fctDraw.Set_Origin_X(System.Convert.ToInt32(txtOMovX.Text)); }
            catch (Exception)
            {
                fctDraw.Set_Origin_X(0);
                txtOMovX.Text = "0";
            }
            //redraw without invalid function
            picDraw.Invalidate();
        }

        private void txtScaleY_TextChanged(object sender, EventArgs e)
        {
            try
            { fctDraw.Set_Scale_Y(System.Convert.ToDouble(txtScaleY.Text)); }
            catch (Exception)
            {
                fctDraw.Set_Scale_Y(20);
                txtaccuracy.Text = "20";
            }
            //redraw without invalid function
            picDraw.Invalidate();
        }

        private void txtScaleX_TextChanged(object sender, EventArgs e)
        {
            try
            { fctDraw.Set_Scale_X(System.Convert.ToDouble(txtScaleX.Text)); }
            catch (Exception)
            {
                fctDraw.Set_Scale_X(20);
                txtScaleX.Text = "20";
            }
            //redraw without invalid function
            picDraw.Invalidate();
        }
        #endregion

        private void inverseYAxisToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (fctDraw.Get_Inverse_Factor() == -1)
            {
                fctDraw.Set_Inverse_Factor(1);
                inverseYAxisToolStripMenuItem.Checked = true;
            }
            else
            {
                fctDraw.Set_Inverse_Factor(-1);
                inverseYAxisToolStripMenuItem.Checked = false;
            }
            //redraw without invalid function
            picDraw.Invalidate();
        }

        private void actualizefunctionsToolStripMenuItem_Click(object sender, EventArgs e)
        { Actualize_Function_Data(); }

        private void frmDrawFunction_Shown(object sender, EventArgs e)
        {
            //Actualize_Function_Data();
            //fctDraw.Draw_Scale(picDraw.CreateGraphics());
        }

        private void picDraw_MouseDown(object sender, MouseEventArgs e)
        {
            //start zoombox
            fctDraw.Set_Zoom_Box_X(e.X);
            fctDraw.Set_Zoom_Box_Y(e.Y);
            fctDraw.Set_Zoom_Box_Width(0);
            fctDraw.Set_Zoom_Box_Height(0);

            if (e.Button == System.Windows.Forms.MouseButtons.Left)
                fctDraw.Set_Mouse_State(MouseState.ZoomBox);
            else if (e.Button == System.Windows.Forms.MouseButtons.Right)
                fctDraw.Set_Mouse_State(MouseState.MoveOrigin);
        }

        //perform view modification actions on mouseup
        private void picDraw_MouseUp(object sender, MouseEventArgs e)
        {
            //final width and height update
            fctDraw.Set_Zoom_Box_Width(e.X - fctDraw.Get_Zoom_Box_X());
            fctDraw.Set_Zoom_Box_Height(e.Y - fctDraw.Get_Zoom_Box_Y());

            //if view modification was not wanted
            if (Math.Abs(fctDraw.Get_Zoom_Box_Width()) < 5 && Math.Abs(fctDraw.Get_Zoom_Box_Height()) < 5)
            {   //reset zoombox
                fctDraw.Set_Mouse_State(MouseState.Nothing);
                return;
            }

            //perform actions
            if (fctDraw.Get_Mouse_State() == FunctionDrawing.MouseState.ZoomBox)            //_________________ZoomBox_____________________
            {
                fctDraw.Resolve_Zoom_Box();

                txtScaleX.Text = fctDraw.Get_Scale_X().ToString();
                txtScaleY.Text = fctDraw.Get_Scale_Y().ToString();
                txtOMovX.Text = fctDraw.Get_Origin_X().ToString();
                txtOMovY.Text = fctDraw.Get_Origin_Y().ToString();

                //redraw without box
                Actualize_OriginAndScale();
                fctDraw.Calc_Functions();
                picDraw.Invalidate();
            }
            else if (fctDraw.Get_Mouse_State() == FunctionDrawing.MouseState.MoveOrigin)    //_________________Move_Origin_____________________
            {
                fctDraw.Resolve_Move_Origin();

                txtOMovX.Text = fctDraw.Get_Origin_X().ToString();
                txtOMovY.Text = fctDraw.Get_Origin_Y().ToString();

                //redraw without arrow
                Actualize_OriginAndScale();
                fctDraw.Calc_Functions();
                picDraw.Invalidate();
            }
        }

        //redraw draw screen
        private void picDraw_Paint(object sender, PaintEventArgs e)
        {
            fctDraw.Draw_All(e.Graphics);
        }

        private void picDraw_MouseLeave(object sender, EventArgs e)
        {
            //remove zoombox
            fctDraw.Set_Mouse_State(MouseState.Nothing);
            //redraw without box
            picDraw.Invalidate();
        }

        //set default values for view modificators
        private void standardViewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            fctDraw.Set_Origin_X(0);
            fctDraw.Set_Origin_Y(0);
            txtOMovX.Text = fctDraw.Get_Origin_X().ToString();
            txtOMovY.Text = fctDraw.Get_Origin_Y().ToString();

            fctDraw.Set_Scale_X(20);
            fctDraw.Set_Scale_Y(20);
            txtScaleX.Text = fctDraw.Get_Scale_X().ToString();
            txtScaleY.Text = fctDraw.Get_Scale_Y().ToString();

            fctDraw.Set_Accurracy(1);
            txtaccuracy.Text = fctDraw.Get_Accurracy().ToString();

            actualizeToolStripMenuItem.PerformClick();
        }

        //if frmdrawfunction is closed show frmMain
        private void frmDrawFunction_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                if (Program.frmMain.WindowState != FormWindowState.Normal)
                    Program.frmMain.WindowState = FormWindowState.Normal;
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// Save picturebox.image in a file of chosen name and format.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void saveGraphToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Bitmap tempBmp = new Bitmap(picDraw.Width, picDraw.Height);
            System.Drawing.Imaging.ImageFormat saveFormat = System.Drawing.Imaging.ImageFormat.Png;

            picDraw.DrawToBitmap(tempBmp, new Rectangle(0, 0, picDraw.Width, picDraw.Height));

            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (saveFileDialog.FilterIndex == 1)                                //bmp
                    saveFormat = System.Drawing.Imaging.ImageFormat.Bmp;
                else if (saveFileDialog.FilterIndex == 2)                           //gif
                    saveFormat = System.Drawing.Imaging.ImageFormat.Gif;
                else if (saveFileDialog.FilterIndex == 3)                           //jpeg
                    saveFormat = System.Drawing.Imaging.ImageFormat.Jpeg;
                //else if (saveFileDialog.FilterIndex == 4)                           //png
                //    saveFormat = System.Drawing.Imaging.ImageFormat.Png;
                else if (saveFileDialog.FilterIndex == 5)                           //tiff
                    saveFormat = System.Drawing.Imaging.ImageFormat.Tiff;

                try
                {
                    tempBmp.Save(saveFileDialog.FileName, saveFormat);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("An Error occurred while saving: " + ex.Message);
                }
            }
        }
    }
}

