﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CalculationE
{   
    /// <summary>
    /// 1 element of a calculation
    /// </summary>
    public class CalculationElement
    {
        /// <summary>
        /// Type of Calculation Element
        /// </summary>
        //parameter is used for calcfunction parameter
        public enum ElementType { Operator, Number, Variable, Function, Invalid, Parameter }
        public ElementType Type;
        private double Value;
        public int Index;
        public int NestedLevel;
        private double sign;                                                //use only if type is not number

        //constructor for numbers
        public CalculationElement(double value, int nestedLevel)
        {
            this.Type = ElementType.Number;
            this.Value = value;
            this.Index = -1;
            this.NestedLevel = nestedLevel;
            this.sign = 1.0;
        }

        /// <summary>
        /// Constructor for other elements. Do not use for numbers.
        /// </summary>
        /// <param name="index"></param>
        /// <param name="eType"></param>
        public CalculationElement(ElementType eType, int index, int nestedLevel)
        {
            this.Index = index;
            this.Type = eType;
            this.Value = -1.0;
            this.NestedLevel = nestedLevel;
            this.sign = 1.0;
        }

        /// <summary>
        /// Constructor for new nested level. Copies all other data.
        /// </summary>
        /// <param name="index"></param>
        /// <param name="eType"></param>
        public CalculationElement(CalculationElement calcE, int nestedLevel)
        {
            this.Index = calcE.Index;
            this.Type = calcE.Type;
            this.Value = calcE.Value;
            this.NestedLevel = nestedLevel;
            this.sign = calcE.sign;
        }

        /// <summary>
        /// Constructor for new element. Copies all data.
        /// </summary>
        /// <param name="index"></param>
        /// <param name="eType"></param>
        public CalculationElement(CalculationElement calcE)
        {
            this.Index = calcE.Index;
            this.Type = calcE.Type;
            this.Value = calcE.Value;
            this.NestedLevel = calcE.NestedLevel;
            this.sign = calcE.sign;
        }

        //does not work: Changes are not safed to this object
        //maybe bug, maybe only because of using lists
        /// <summary>
        /// returns a new calcElement with inverted sign
        /// </summary>
        /// <returns></returns>
        public CalculationElement Negate()
        {
            CalculationElement calcE = new CalculationElement(this);

            calcE.sign = this.Sign() * -1.0;

            return calcE;
        }
        /*
        static public void Negate(CalculationElement calcE)
        {
            calcE.sign *= -1.0;
        }
        static public CalculationElement Negate(CalculationElement calcE)
        {
            calcE.sign *= -1.0;
            return calcE;
        }
        */

        public double Sign()
        {
            return this.sign;
        }

        public double GetValue()
        {
            return this.sign * this.Value;
        }

        public Int32 ToInt()
        {
            return System.Convert.ToInt32(Math.Round(this.sign * this.Value));
        }

        public bool ToBool()
        {
            if (this.Value == 0)
                return false;
            else
                return true;
        }

        public void SetValue(double val)
        {
            if (val < 0.0)
            {
                this.sign = -1.0;
                this.Value = Math.Abs(val);
            }
            else
            {
                this.sign = 1.0;
                this.Value = val;
            }
        }

        /// <summary>
        /// Give operand; return interpreted value of operand as CalculationElement
        /// </summary>
        /// <param name="operand"></param>
        /// <returns></returns>
        public CalculationElement Interprete_Variable()
        {
            if (this.Type == ElementType.Variable)
            {

                double tempVal = CalcVariables.Get_Var(this.Index) * this.sign;

                return new CalculationElement(tempVal, NestedLevel);
            }
            else
            {
                return this;
            }
        }

        //return string of what is currently saved in this element
        public override string ToString()
        {
            switch (this.Type)
            {
                case ElementType.Function:
                    if (this.sign > 0)
                        return CalcFunctions.Get_Function_Name(this.Index);
                    else
                        return "-" + CalcFunctions.Get_Function_Name(this.Index);
                case ElementType.Number:
                    return (this.Value * this.Sign()).ToString();
                case ElementType.Invalid:
                    if (this.sign > 0)
                        return "Invalid element";
                    else
                        return "-" + "Invalid element";
                case ElementType.Operator:
                    if (this.sign > 0)
                        return CalcOperators.opCompareStr[this.Index];
                    else
                        return "-" + CalcOperators.opCompareStr[this.Index];
                case ElementType.Parameter:
                    if (this.sign > 0)
                        return "x" + this.Index.ToString();
                    else
                        return "-" + "x" + this.Index.ToString();
                case ElementType.Variable:
                    if (this.sign > 0)
                        return CalcVariables.Get_Var_Name(this.Index);
                    else
                        return "-" + CalcVariables.Get_Var_Name(this.Index);
                default:
                    return "Invalid type";
            }
        }

        //return name of type
        public string TypeToString()
        {
            switch (this.Type)
            {
                case ElementType.Function:
                    return "Function";
                case ElementType.Number:
                    return "Number";
                case ElementType.Invalid:
                    return "Invalid";
                case ElementType.Operator:
                    return "Operator";
                case ElementType.Parameter:
                    return "Parameter";
                case ElementType.Variable:
                    return "Variable";
                default:
                    return "Invalid type";
            }
        }
    }

}
